#!/bin/sh
# This comment continues on in Tcl, but not in sh \
exec wish "$0" ${1+"$@"}

# Default values for control variables
if {![info exists no_net]} {set no_net 0}
if {![info exists no_event_loop]} {set no_event_loop 0}

# Command line options
if {[string match "--nonet" [lindex $argv 0]]} {
    set no_net 1
}

# Directory knowledge
global mmpe
set mmpe(dirrel) [string trimright [file dirname [info script]] ./]
set mmpe(dirabs) [file join [pwd] $mmpe(dirrel)]
set mmpe(libdir) [file join [file dirname $mmpe(dirabs)] oommf library]

# Make sure oommf.tcl is sourced
source [file join $mmpe(libdir) oommf0.0 oommf.tcl]

#*************************************************************************#
#ReadFile read a file into an input  buffer.  Does not parse the file  
proc ReadFile {filename } {
 puts "Reading $filename..."
 global inbuff
 set inbuff [list]
 set count 0 
 if {[file readable $filename]} {
  set file1Id [open $filename "r"]
    while { [gets $file1Id data]>=0} {
      #puts $data
      lappend inbuff $data
      incr count
    }
  close $file1Id
 }
if {$count == 0} {puts stderr "Empty file or file not readable"}
return $count
}   
#*************************************************************************#
proc ChangeMif { Ref NewValue } {
    global MifKnow
    set MifKnow($Ref) [concat [lrange $MifKnow($Ref) 0 0] $NewValue]
}

#*************************************************************************#
#set up an array that contains the entries of a MifFile
#What is needed in a mif file is stored in state.dat
proc KnowWhat {filename} {
 global inbuff MifKnow KnowOrder
 set listsize [ReadFile $filename]
 for {set i 0} {$i<$listsize} {incr i} {
  set shortbuff [lindex $inbuff $i] 
  #puts $shortbuff
  set shortbuff [split $shortbuff :]
  set MifKnow([lindex $shortbuff 0]) [lindex $shortbuff 1]
  set KnowOrder([lindex $shortbuff 2]) [lindex $shortbuff 0]
 }
}
  #Reads and understands the mif file <filename> (must run KnowWhat first) 
  #(needs ReadFile) puts result in array MifKnow
#*************************************************************************#
proc ParseFile {filename} {
    global inbuff MifKnow FieldState
    set listsize [ReadFile $filename]
    set firstbuff vals
    set FieldCount 0 
    set prefix {} ; set suffix {}
    for {set i 0} {$i<$listsize} {incr i} {
	set shortbuff [string trim [lindex $inbuff $i]]
	regexp -nocase {^[^#]*[^#]*} $shortbuff firstbuff
	if {[string length $firstbuff]!=0} {
	    #set firstbuff [string tolower $firstbuff]
	    if {![regexp -- {^([^:=]+)([:=]+)(.+)} $firstbuff fluf \
		    prefix fluf2 suffix]} {
		error "<dmif.tcl:Parsefile>:\
			Incomprehensible input line: $shortbuff"
	    }
	    if {[string match {} $prefix] || [string match {} $suffix]} {
		error "<dmif.tcl:Parsefile>: Invalid input line: $shortbuff"
	    }
	    set prefix [string tolower [string trim $prefix]]
	    set suffix [string trim $suffix]
	    if [info exists MifKnow($prefix)] { 
		if {$prefix=="field range"} {
		    incr FieldCount
		    set FieldState($FieldCount) $suffix
		    #puts "$FieldState($FieldCount) $FieldCount"
		}
		ChangeMif $prefix $suffix
	    }
	}
    }
}

#*************************************************************************#
proc CallBack {filename num} {
    global MifKnow MifBak                                    
    foreach i [array names MifBak] {set MifKnow($i) $MifBak($i)}
}
#*************************************************************************#
proc CallNew {num} {
    global TopButton
    set size [expr [array size TopButton]-1] 
    if {$num<0} {set num $size}
    if {$num>$size} {set num 0}                  
    set filename $TopButton($num)
    EntryChange $filename $num
}
#*************************************************************************#
proc MultiButton {type name label label2 btext bchange width1 width2 command command2 args} {
     #puts "t:$type n:$name l:$label l2:$label2 b:$btext bch:$bchange w:$width1 w2:$width2 c:$command c2:$command2 args: $args"
     global v vals val
     frame $name
     
     #event add <<INVOKE>> <Shift-F1>
     set a ".[lindex [split $name .] 1].[lindex [split $name .] 2]"
     if {$type==0} {
      #check only 
      radiobutton $name.button -text $bchange \
                  -value $btext -command $command2 -anchor w
      pack $name.button -side left 
     }
     if {$type==1} {
      #just a label
      label $name.label -text $label -anchor w
      pack $name.label -side left
     }
     if {$type==2} {
      #label and radiobutton
      label $name.label -text "$label:" -anchor e
      radiobutton $name.button -text\
                  $bchange -value $btext -command $command2
      pack $name.label -side left
      pack $name.button -side right
     }
     if {$type==3} {
      #check + label + entry
	radiobutton $name.button -text $bchange \
		-value $btext -command $command2 
	label $name.label -text "$label:" -width $width1 -anchor e
	eval  {entry $name.entry -width $width1 -relief sunken} $args
	pack $name.button -side left
	pack $name.label -side left
        pack $name.entry -side right -fill x -expand 1
	$name.entry config -state disabled
	#bind $name.entry <<INVOKE>> $command
	bind $name.entry <Return> "$command;focus $a.next.ok;break"
	bind $name.entry <Tab> $command	
     }
     if {$type==4} {
      #label + entry wigit
	label $name.label -text "$label:" -width $width1 -anchor e
	eval  {entry $name.entry -relief sunken} $args
	pack $name.label -side left
	pack $name.entry -side right -fill x -expand 1
        #bind $name.entry <<INVOKE>> $command
	bind $name.entry <Return> "$command;focus $a.next.ok;break"
	bind $name.entry <Button-1> "Set $name"
	bind $name.entry <Tab> $command
	
     }
	if {$type==5} {
      #check + label + entry + label + entry 
	radiobutton $name.button -text\
                  $bchange -value $btext -command $command2 
	label $name.label -text "$label:" -width $width1 -anchor w
	eval  {entry $name.entry -relief sunken -width $width1} $args
	label $name.label2 -text "$label2:" -width $width1 -anchor w
	eval  {entry $name.entry2 -relief sunken -width $width1} $args
	pack $name.button -side left
	pack $name.entry $name.label $name.entry2 $name.label2 -side right
	bind $name.entry <Tab> $command
        #bind $name.entry <<INVOKE>> $command
	bind $name.entry <Return> "$command;focus $a.next.ok;break"
	bind $name.entry <Button-1> "Set $name"
	bind $name.entry2 <Tab> $command
        #bind $name.entry2 <<INVOKE>> $command
	bind $name.entry2 <Return> "$command;focus $a.next.ok;break"
	bind $name.entry2 <Button-1> "Set $name"
	$name.entry config -state disabled
 	$name.entry2 config -state disabled
     }
  if {$type==6} {
   set ButtonState $name.entry
	set textind [split $label ,]
      #N label + entries
        set lwidth [string length "[lindex $textind 0]:"]
	label $name.label -text "[lindex $textind 0]:" -width $lwidth -anchor e
	eval  {entry $name.entry -relief sunken -width $width1}
	pack $name.label -side left
	pack $name.entry -side left -fill x -expand 1
	#bind $name.entry <<INVOKE>> $command
	bind $name.entry <Return> "$command;focus $a.next.ok;break"
	bind $name.entry <Button-1> "Set $name"
	bind $name.entry <Tab> $command
	set b2 ${name}.entry
	$b2 config -textvar val($b2) 
  	for {set j 2} {$j<$label2} {incr j} { 
	 set b2 ${name}.entry$j
	 set b3 ${name}.label$j	 
         set lwidth [string length "[lindex $textind [expr $j-1]]:"]
	 label $b3 -text "[lindex $textind [expr $j-1]]:" -width $lwidth -anchor e
	 eval  {entry $b2 -relief sunken -width $width1}
	 pack $b3 -side left
	 pack $b2 -side left -fill x -expand 1
	 $b2 config -textvar val($b2)
	 puts "texvarname $b2"
	 bind $b2 <Tab> $command
         #bind $b2 <<INVOKE>> $command
	 bind $b2 <Return> "$command;focus $a.next.ok;break"
	 bind $b2 <Button-1> "Set $name"
        }}
 return $name.entry
}
#*************************************************************************#
proc Set {name} {
    set a [focus -lastfor $name]
    puts "Inside of set: $a"
    eval [bind $a <Tab>]
    #event generate $a <<INVOKE>>
}
#*************************************************************************#
proc ScaleButton {type name labs label2 btext ButtonNow width1 width2
                  command command2 args} {
      global v vals val Dchange MifKnow
      #puts "type: $type  $name $labs  $ButtonNow w1: $width1 w2: $width2 "
      puts $Dchange($ButtonNow)
      #set vals($ButtonNow) MifKnow($Dchange($ButtonNow))
      frame $name
	set b2 ${name}.scale
      #label $b2 -text "Organic"
      #set $vals($ButtonNow) 2
        scale $b2 -from 0 -to 10 -length 600 -variable vals($ButtonNow) \
	 -orient horizontal -label $labs \
	 -tickinterval 3 -showvalue true \
      -command  "NewField $type $name $ButtonNow $width1 $width2 \
                $command $command2 $args"
      pack $b2 -fill x -expand 1
}
#*************************************************************************#
proc NewField {type name ButtonNow width1 width2 command command2 args}  {
 global MifKnow Dchange vals val FieldState
 if {[info commands ${name}.2]!=""} {destroy ${name}.2}
 if {$vals($ButtonNow)==0} {
     set MifKnow($Dchange($ButtonNow)) "1"
 } else {
     set MifKnow($Dchange($ButtonNow)) "1 $vals($ButtonNow)"
 }
 puts "datanow: $MifKnow($Dchange($ButtonNow))"
 frame ${name}.2
 for {set j 1} {$j<=$vals($ButtonNow)} {incr j} {
    set b2 ${name}.2.$j
    MultiButton $type $b2 "x,y,z,x,y,z,steps" 8 4 $ButtonNow $width1 \
                $width2 "NewVal3 $ButtonNow $j \
                ${b2}.entry" "" "-textvar vals($ButtonNow)"
    set val($b2) "0.0"
    pack $b2
    if { [info exists FieldState($j)]} {
         puts $FieldState($j) 
    } else {
       set FieldState($j) "0 0 0 0 0 0 1" } 
    puts $FieldState($j) 
    set k 1
    set bname $b2
    set b3 $b2.entry 
    while {[info commands $b3]!=""} {
         set val($b3) [lindex $FieldState($j) [expr $k-1]]
	   #puts $f "Mnow: $MNow: [lindex $MNow $ii]:$k:$b2"
         incr k
         set b3 ${bname}.entry$k 
    } 
    Set $b2  
 }
 pack  ${name}.2 -side bottom
 pack $name -side top -fill x
}
#*************************************************************************#
proc NewVal { ButtonNow }  {
 global MifKnow Dchange vals
 set i 1
 set j 1
   set ListElms [llength $MifKnow($Dchange($ButtonNow))]
   
   if {$ListElms>$j} {                                             
      set MifKnow($Dchange($ButtonNow)) \
      [lreplace $MifKnow($Dchange($ButtonNow)) $j $j $vals($ButtonNow)]                       
   } else {                                                        
     lappend MifKnow($Dchange($ButtonNow)) $vals($ButtonNow)                    
   }                      
    puts $MifKnow($Dchange($ButtonNow))
}
#*************************************************************************#
proc NewVal3 { ButtonNow offset ButtonState} {
 global FieldState Dchange val MifKnow
 set ButtonRef $ButtonState
 set i 1
 set j 0
 set fv $offset 
 #[lrange $flipped 1 end]
 puts "$fv"
 if { [info exists FieldState($fv)]} {
   puts $FieldState($fv) 
} else {
   set FieldState($fv) "0 0 0 0 0 0 1" }  
 while {[info commands $ButtonRef]!=""} {
   incr i   
   set ListElms [llength $FieldState($fv)]
   puts "$ListElms $val($ButtonRef) $ButtonRef" 
   if {$ListElms>$j} {                                             
      set FieldState($fv) [lreplace $FieldState($fv) $j $j $val($ButtonRef)]   
                    
   } else {                                                        
     lappend FieldState($fv) $val($ButtonRef)                    
   }                      
    incr j
    set ButtonRef ${ButtonState}$i
 }	   
}
#*************************************************************************#
proc ListButtons {type name filename} {
     global v vals val Dchange MifKnow MatTypes mmpe
     puts "Reading $filename..."
     set input [list]
     set count 0 
     set filename [file join $mmpe(dirabs) $filename]
     if {[file readable $filename]} {
     set file1Id [open $filename "r"]
     while { [gets $file1Id data]>=0} {
       #puts $data
       lappend input $data
       puts $data
       incr count
     }
     close $file1Id
     }
     if {$count == 0} {puts "Empty file or file not readable"}
     frame $name -relief raised
     set b2 ${name}.list
     menubutton $b2 -text "Material Types" -menu ${b2}.menu\
            -relief raised -underline 0
     pack $b2 -side left -padx 10 -pady 10
     set b2 $name
     button ${b2}.add -text "Add" -command "puts add" -state disabled
     button ${b2}.replace -text "Replace" -command "puts replace" -state disabled
     button ${b2}.delete -text "Delete" -command "puts Delete" -state disabled
    
     pack ${b2}.add ${b2}.replace ${b2}.delete -side left -fill x
     set b2 ${name}.list.menu
     set m1 [menu $b2 -tearoff 0]
     set iteration 0
     set NULL 0
     while {$iteration<$count} {
       set locallist [lindex $input $iteration]
       puts $locallist
       split $locallist
       set Name [lindex $locallist 0]
       set Ms [lindex $locallist 1]
       set a [lindex $locallist 2]
       set k [lindex $locallist 3]
       set anistropy [lindex $locallist 4]
       set MatTypes($Name) [list $Name $Ms $a $k $anistropy]
       $m1 add radio -label $Name -variable $NULL -command "SetMaterial $Name"
       incr iteration
    }
    DetMaterial
    #pack $b2 -side top -fill x
}   
#*************************************************************************#
proc SetMaterial {name} {
 global MatTypes vals
 puts "set material: $MatTypes($name)"
       set locallist [split $MatTypes($name)]  
       set vals(2) [lindex $locallist 0];NewVal 2
       set vals(3) [lindex $locallist 1];NewVal 3
       set vals(4) [lindex $locallist 2];NewVal 4
       set vals(5) [lindex $locallist 3];NewVal 5
       set atype [lindex $locallist 4]
       if { [string compare $atype uniaxial]==0 } {
          .change.mats.6.uniaxial.button invoke
       } elseif {  [string compare $atype cubic]==0} {
          .change.mats.6.cubic.button invoke
       }
       puts $vals(2)	
}
#*************************************************************************#
proc DetMaterial { } {
global MatTypes MifKnow vals Dchange
puts "Inside DetMaterials"
if [info exists vals(2)] {
 } else {
   set vals(2) "Custom";set Dchange(2) "material name";NewVal 2}
foreach name [array names MatTypes] {
       if {$name==$vals(2)} {
       set vals(2) "Custom";set Dchange(2) "material name";NewVal 2}
       set locallist [split $MatTypes($name)]  
       set ms [lindex $locallist 1]
       set a [lindex $locallist 2]
       set k [lindex $locallist 3]
       set ms2 [lindex $MifKnow(ms) 1]
       set a2 [lindex  $MifKnow(a) 1] 
       set k2 [lindex  $MifKnow(k1) 1]
       if {$ms2!=""&&$a2!=""&&$k2!=""} {
       puts "$ms:$ms2 $a:$a2 $k:$k2"
       if {[expr $ms==$ms2]&&\
           [expr $a==$a2]&&\
           [expr  $k==$k2]} {
       set vals(2) $name;set Dchange(2) "material name";NewVal 2 }}
}
}  
#*************************************************************************#
proc ChangeMaterials {function} {
global MatTypes
if { [string compare $function delete]==0} {
   }}
#*************************************************************************#
proc NewVal2 { ButtonNow offset ButtonState} {
 global MifKnow Dchange val
 set ButtonRef $ButtonState
 puts " $MifKnow($Dchange($ButtonNow)) "
 set i 1
 set j $offset
 while {[info commands $ButtonRef]!=""} {
   incr i   
   set ListElms [llength $MifKnow($Dchange($ButtonNow))]
   puts "$ListElms $val($ButtonRef) $ButtonRef" 
   if {$ListElms>$j} {                                             
      set MifKnow($Dchange($ButtonNow)) \
      [lreplace $MifKnow($Dchange($ButtonNow)) $j $j $val($ButtonRef)]                       
   } else {                                                        
     lappend MifKnow($Dchange($ButtonNow)) $val($ButtonRef)                    
   }                      
    incr j
    set ButtonRef ${ButtonState}$i
 }	   
}
#*************************************************************************#
proc DisableEntry { ButtonNow x} {
  global MifKnow Dchange ButtonState
   set b2 ""
   set NULL ""
   if {$ButtonState!=$NULL} {
    $ButtonState config -textvar $NULL
    set last [$ButtonState index end]
    $ButtonState delete 0 $last 
    $ButtonState config -state disabled
    set b2 ${ButtonState}2
   }
   if {[info commands $b2]!=""} {
     $b2 config -textvar $NULL
     set last [$b2 index end]
     $b2 delete 0 $last 
     $b2 config -state disabled

   }
}
#*************************************************************************#
proc ChangeMif2 { Ref ButtonNow Command x} {
 global MifKnow Dchange ButtonState
 set ListElms [llength $MifKnow($Dchange($ButtonNow))]
 focus .change
 if {$ListElms>=2} {
   set MifKnow($Dchange($ButtonNow)) \
       [lreplace $MifKnow($Dchange($ButtonNow)) 1 $ListElms $Ref]
 } else {
   lappend MifKnow($Dchange($ButtonNow)) $Ref
 }
 if {$Command!="NULL"} {
  DisableEntry $Command $x
 }
 set ListElms [llength $MifKnow($Dchange($ButtonNow))]
 puts " $ListElms $MifKnow($Dchange($ButtonNow)) "
} 
#*************************************************************************#
proc ChangeMif3 { Ref ButtonNow name type} {
 global MifKnow Dchange ButtonState val
 DisableEntry $ButtonNow ""
 puts "changing mif"
 set ListElms [llength $MifKnow($Dchange($ButtonNow))]
 if {$ListElms>=2} {
   set MifKnow($Dchange($ButtonNow)) \
       [lreplace $MifKnow($Dchange($ButtonNow)) 1 $ListElms $Ref]
 } else {
   lappend MifKnow($Dchange($ButtonNow)) $Ref
 }
 set ButtonState $name.entry
 set b2 $ButtonState
 set i 1
 while {[info commands $b2]!=""} {
   $b2 config -state normal
   set val($b2) "0.0"
   $b2 config -textvar val($b2)
   incr i   
   lappend MifKnow($Dchange($ButtonNow)) $val($b2)
   set b2 ${ButtonState}$i 
 }
 focus $name.entry 
 
}
#*************************************************************************#
proc SplitNToss {filename} {
 global inbuff MenuState
 set MenuState [list]
 puts $filename
 set listsize [ReadFile $filename]
 set firstbuff vals
 set FieldCount 0
 for {set i 0} {$i<$listsize} {incr i} {
   set shortbuff [string trim [lindex $inbuff $i]]
   regexp -nocase {^[^#]*[^#]*} $shortbuff firstbuff
   if {[string length $firstbuff]!=0} {
      set firstbuff [string trim [lindex [split $firstbuff :] 1 ]]
      #puts $firstbuff
      if {[string length $firstbuff]!=0} {
	  lappend MenuState $firstbuff
      }
   }
 }
}
#*************************************************************************#
proc EntryChange {filename ButtonNumber} {
 global inbuff MifKnow Mcontents val vals\
        Dchange MenuState ButtonState MifBak TopButton 
 set ButtonState ""
 set NULL "NULL"
 set size [expr [array size TopButton]-1]
 set prev [expr $ButtonNumber-1]
 set next [expr $ButtonNumber+1]
 foreach i [array names MifKnow] {set MifBak($i) $MifKnow($i)}      
 SplitNToss $filename
 #puts $MenuState
 #sets name of window
 wm geometry .change {}  ;# Display window at widget requested size
 set a .change[lindex $MenuState 0]
 frame $a ;# This frame fills toplevel .change
 after idle [list wm title .change \
	 "mmProbEd<[pid]>: [file rootname [file tail $filename]]"]
 #determines number of commands
 set ComNum [lindex $MenuState 1]
 #reads commands and options into a command array
 set i 2
 for {set Com 0} {$Com<$ComNum} {incr Com} {
   set comands($Com) [lindex $MenuState $i]
   incr i 
   set options($Com) [lindex $MenuState $i]
   incr i
 }
 #reads number of data changes and sets up frames
 set Types [lindex $MenuState $i]
 # variable Types holds number of mif changes in menu
 set ManyOld 0
 incr i
 for {set ButtonNow 1} {$ButtonNow<=$Types} {incr ButtonNow} {
  frame $a.$ButtonNow
  #Dchange hold mif changes; can be used with MifKnow
  set Dchange($ButtonNow) [lindex $MenuState $i]
  puts "$Dchange($ButtonNow) $ButtonNow"
  set loopn $Dchange($ButtonNow)
  incr i
  set vals($ButtonNow) [lindex $MifKnow($Dchange($ButtonNow)) 1]
  set MNow $MifKnow($Dchange($ButtonNow))
  #entries hold the number of options for a given MifChange
  set entries [lindex $MenuState $i]
  incr i
  set OptionMax [expr $entries + $ManyOld]   
  for {set j $ManyOld} {$j<$OptionMax} {incr j} {
    set shortbuff [split [lindex $MenuState $i] ! ]
    #Options Holds name & value of Menu choice    
    set Options($j) [lindex $shortbuff 0]
    incr i
    set b [lindex $shortbuff 0]
    set c [lindex $shortbuff 1]
    set type [lindex $shortbuff 2]
    set labname [lindex $shortbuff 3] 
    set labname2 [lindex $shortbuff 4]
    if {[lindex $shortbuff 5]==""} {
      set offset 1 
    } else {
      set offset [lindex $shortbuff 5]
    }
    
    set name [lindex $b 0]
    join $b
    eval $comands($c)
    set bname $a.$ButtonNow.$name 
    set cname "pack $bname $options($c)"
    eval $cname
    set check [string tolower [split [lindex $vals($ButtonNow) 0]]]
 
    if {$check==$b} {
	 set b2 $a.$ButtonNow.$name.button
	 if {[info commands $b2]!=""} { 
        set cname  "$b2 invoke"
	puts $cname 
        eval $cname
       }
    }
         set ii $offset
	 set k 1
	 set b2 $bname.entry	 
	 while {[info commands $b2]!=""} {
         set val($b2) [lindex $MNow $ii]
	   #puts $f "Mnow: $MNow: [lindex $MNow $ii]:$k:$b2"
         incr ii 
         incr k
         set b2 ${bname}.entry$k 
       }  }
 pack $a.$ButtonNow -fill x -expand 0
 }
 frame $a.next
 button $a.next.next -text "Next" -command "Set $a; destroy $a ; CallNew $next"
 button $a.next.prev -text "Previous" -command "Set $a; destroy $a ; CallNew $prev"
 button $a.next.reset -text "Cancel" \
        -command "destroy .change ; [list CallBack $filename $ButtonNumber]"
 button $a.next.ok -text "Ok" -command "Set $a; destroy .change"
 if {$prev<0} { $a.next.prev configure -state disabled }
 if {$next>$size} { $a.next.next configure -state disabled }
 pack $a.next.ok $a.next.reset $a.next.next $a.next.prev -side left -expand true
 pack $a.next -side bottom -fill x
 bind $a.next.ok <Return> "destroy .change"
 bind $a.next.reset <Return> "destroy .change ; \
	 [list CallBack $filename $ButtonNumber]"
 pack $a -fill both -expand 1
 }
#*************************************************************************#
proc SaveMifFile {savefilename} {
 global MifKnow FieldState KnowOrder
 set file2Id [open $savefilename "w"]
 puts $file2Id "# MIF 1.1"
 for {set one 1} {$one<[array size KnowOrder]} {incr one} { 
    set index $KnowOrder($one)
    set flipped $MifKnow($index)
    set vals [lindex $flipped 1]
    if {$vals!="" && $index!="field range"&&$index!="material name"} {
	puts $file2Id "$index:[lrange $flipped 1 end]"
        puts "l32:$index:[lrange $flipped 1 end]"
    } elseif {$index=="material name"&&$vals!=""} {
        puts $file2Id "# $index:[lrange $flipped 1 end]"
        puts "l31: $index:[lrange $flipped 1 end]"
    }
   
    if {$index=="field range count"} {
     for {set iiii 1} {$iiii <= $vals} {incr iiii} {
      set pals $FieldState($iiii)
       puts "vals: $vals $iiii"
      puts $file2Id "field range: $pals"
      puts "field range: $pals"
     }
    }
 }
 close $file2Id
}
#**************************************************************************#  
# Library support
lappend auto_path $mmpe(libdir)
if {$tcl_version < 7.6} {
    lappend auto_path [file join $mmpe(libdir) em0.0]
    lappend auto_path [file join $mmpe(libdir) oommf0.0]
    if {!$no_net} { lappend auto_path [file join $mmpe(libdir) net0.0] }
}
package require em
package require oommf
if {!$no_net} { package require net }
proc SaveButton { btn item } {
    if { [string match disabled [$btn entrycget $item -state]] } {
        return
    }
    set dialog [omf_mmpesavedlg new -callback SaveCallback \
            -delete_callback_arg \
	    [list $btn entryconfigure $item -state normal]]
    $btn entryconfigure $item -state disabled
}
proc SaveCallback { widget actionid args } {
    if {[string match DELETE $actionid]} {
        eval [join $args]
        return
    }
    if {[string match SAVE $actionid]} {
        SaveMifFile [$widget GetFilename]
    } else {
        return "ERROR (proc SaveCallback): Invalid actionid: $actionid"
    }
}
proc LoadButton { btn item } {
    if { [string match disabled [$btn entrycget $item -state]] } {
        return
    }
    set dialog [omf_mmpeopendlg new -callback LoadCallback \
            -delete_callback_arg \
	    [list $btn entryconfigure $item -state normal]]
    $btn entryconfigure $item -state disabled
}
proc LoadCallback { widget actionid args } {
    global resetfilepath
    if {[string match DELETE $actionid]} {
        eval [join $args]
        return
    }
    if {[string match OPEN $actionid]} {        
	KnowWhat $resetfilepath
        ParseFile [$widget GetFilename]
    } else {
        return "ERROR (proc LoadCallBack): Invalid actionid: $actionid"
    }
}
#**************************************************************************#
proc CallWin {procedurename filename windownumber} {
    if {[winfo exists .change]} { destroy .change }
    toplevel .change 
    after idle "OMF_SetIcon .change"
    wm title .change "mmProbEd<[pid]>: [file rootname [file tail $filename]]"
    $procedurename $filename $windownumber 
}
#**************************************************************************#  
#top level window
frame .menubar -borderwidth 3 -relief raised
frame .menubuttons -borderwidth 4 -relief ridge
menubutton .menubar.file -text "File" -menu .menubar.file.menu \
-underline 0
menubutton .menubar.help -text "Help" -menu .menubar.help.menu \
-underline 0 -state disabled
pack .menubar.file -side left
pack .menubar.help -side right
set menuname [menu .menubar.file.menu -tearoff no]
$menuname add command -label "Open..." -command\
           [list LoadButton $menuname "Open..."] -underline 0
$menuname add command -label "Save as..." -command\
           [list SaveButton $menuname "Save as..."] -underline 0
$menuname add separator
$menuname add command -label Exit -command {Die} -underline 1
global resetfilepath
KnowWhat [file join $mmpe(dirabs) state.dat]
set resetfilepath [file join $mmpe(dirabs) state.dat]
puts $resetfilepath
ParseFile [file join $mmpe(dirabs) init.mif]
global TopButton
set listsize [ReadFile [file join $mmpe(dirabs) topmenu]]
for {set i 0} {$i<$listsize} {incr i} {
    set shortbuff [split [lindex $inbuff $i] : ]

    set field0 [string trim [lindex $shortbuff 0]]
    set field1 [string trim [lindex $shortbuff 1]]
    set field2 [string trim [lindex $shortbuff 2]]

    set buttoname [string tolower ".menubuttons.$field0"]
    set buttonspecs($i) $field1
    set filename $field2
    set fullfilename [file join $mmpe(dirabs) $filename]
    set TopButton($i) $fullfilename
    button $buttoname -text $field0 \
	    -command [list CallWin $field1 $fullfilename $i]
    if {![file readable $fullfilename]} {
	$buttoname configure -state disabled
    }
    pack $buttoname  -fill x
}
pack .menubar -side top -fill x -expand 1
pack .menubuttons -side bottom -fill x -expand 1 
#
update idletasks
proc Die {} {
    global protocol
    if {[info exists protocol]} {
	$protocol delete; after 2000 exit
    } else {
	exit
    }
}
bind . <Destroy> "+Die"


#**************************************************************************#  
# networking
if {!$no_net} {
    set protocol [net_protocol new -name "OOMMF ProbEd protocol 0.0"]
    $protocol AddMessage start HasGui {} {
	return [list start [list 0 0]]
    }
    $protocol AddMessage start GetProbDescFile {} {
	set fn [OMF_TempName {} tmp ".mif"]
        SaveMifFile $fn
        return [list start [list 0 $fn]]
    }
    set server [net_server new -protocol $protocol \
	    -alias mmProbEd -log em_nop]
    $server Start 0
}

#.quit configure -command { Die } 
wm title . mmProbEd<[pid]>
wm iconname . [wm title .]
## Docs say this should happen automatically, but at least some
## WM's don't track this.

raise .     ;# Bring to top of display window stack

if {!$no_event_loop} { vwait omf_forever }



















