/* FILE: oommf.cc                 -*-Mode: c++-*-
 *
 *      main, WinMain, and Tcl_AppInit for the Oommf application.
 *
 */

#include "machine.h"	/* SYSTEM_TYPE */
#include "misc.h"	/* SCRATCH_BUF_SIZE */
#include "oommf.h"	/* Oommf_Init */

#include <tcl.h>	/* TCL_OK, ... */
#include <tk.h>		/* Tk_Main, ... */
#include <stdio.h>	/* freopen */
#include <stdlib.h>	/* strcmp (?), ... */

/* End includes */     // This is an optional directive to build.tcl,
                       // that excludes the remainder of the file from
                       // '#include ".*"' dependency building

static CHAR buf[SCRATCH_BUF_SIZE];

static char omfIP[] =
"if {[string match 7.6 $tcl_version] && ![info exists tk_library]} {\n\
   set tk_library [file join [file dirname $tcl_library] tk4.2]\n\
 }";

int Tcl_AppInit(Tcl_Interp *interp)
{
#if (SYSTEM_TYPE == WINDOWS)
  // On Windows, use a MessageBox to report initialization errors
  Tcl_SetPanicProc(&Omf_WinPanic);
#endif
    if (Tcl_Init(interp) == TCL_ERROR) {
       panic("Tcl initialization error:\n\n%s\n\n\
As a workaround, set the environment variable\n\
TCL_LIBRARY to the name of a directory\n\
containing an error-free init.tcl.\n\n\
For more information see the INSTALL file.", interp->result);
    }
    if (Tcl_Eval(interp, omfIP) != TCL_OK) {
       panic("Tcl/Tk initialization error:\n\n\
Unable to patch installation bug in Tk 4.2\n\n\
Patch script returned error:%s", interp->result);
    }
    if (Tk_Init(interp) == TCL_ERROR) {
       panic("Tk initialization error:\n\n%s\n\n\
As a workaround, set the environment variable\n\
TK_LIBRARY to the name of a directory\n\
containing an error-free tk.tcl.\n\n\
For more information see the INSTALL file.", interp->result);
    }
    Tcl_StaticPackage(interp, "Tk", Tk_Init, (Tcl_PackageInitProc *) NULL);

//Tcl_GlobalEval(interp,"bgerror \"Tcl/Tk init done!\"");

// Oommf's own initialization
    if (Oommf_Init(interp) == TCL_ERROR) {
	panic("Oommf init error:\n\n%s", interp->result);
    }

//Tcl_GlobalEval(interp,"bgerror \"Oommf init done!\"");

// Start an Oommf User Interface
//    strncpy(buf,"_OMF_StartGui",sizeof(buf));
//    if(Tcl_GlobalEval(interp,buf) != TCL_OK) {
//        panic("Unable to create user interface:\n\n%s", interp->result);
//    }

//panic("Initialization complete!");

    return TCL_OK;

}

#if (SYSTEM_TYPE==WINDOWS)
void
Omf_WinPanic TCL_VARARGS_DEF(char *,arg1)
{
  va_list argList;
  char buf[4096];
  char *format;

  format = TCL_VARARGS_START(char *,arg1,argList);
  vsprintf(buf, format, argList);

  MessageBeep(MB_ICONEXCLAMATION);
  MessageBox(NULL, buf, "Message from mmDisp",
      MB_ICONSTOP | MB_OK | MB_TASKMODAL | MB_SETFOREGROUND);
  ExitProcess(1);
}

int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,
		     LPSTR lpszCmdLine,int nCmdShow)
{
  int argc;
  char *argv[1];
  argv[0]=new char[strlen("oommf")+1];
  strcpy(argv[0],"oommf");

  argc=1; /* Tk_Main will try to input command line arg's as input files */
  Tk_Main(argc,argv,Tcl_AppInit);

panic("Tk_Main returned!");
  delete argv[0];
  return 0;
}
#elif (SYSTEM_TYPE == UNIX)
int main(int argc,char **argv)
{

#if DEBUGLVL<1
  if(fork()==0) {
    // Child process
    freopen("/dev/null","r",stdin);
    freopen("/dev/null","r",stdout);
#endif // DEBUGLVL<1
    argc=1; /* Tk_Main will try to input command line arg's as input files */
    Tk_Main(argc,argv,Tcl_AppInit);
#if DEBUGLVL<1
  }
#endif // DEBUGLVL<1

  return 0;
}
#else
#error "Unknown SYSTEM_TYPE"
#endif // SYSTEM_TYPE == WINDOWS
